@@ -61,6 +61,7 @@
 #endif
 
 #ifdef USE_DNS
+#include <netinet/in.h>
 #include <arpa/inet.h>
 
 #ifdef HAVE_DB_185_H
@@ -99,9 +100,9 @@ char    *our_gzgets(gzFile, char *, int);           /* our gzgets          */
 /* GLOBAL VARIABLES                          */
 /*********************************************/
 
-char    *version     = "2.00";                /* program version          */
-char    *editlvl     = "12";                  /* edit level               */
-char    *moddate     = "15-Apr-2000";         /* modification date        */
+char    *version     = "2.01";                /* program version          */
+char    *editlvl     = "05";                  /* edit level               */
+char    *moddate     = "06-Oct-2000";         /* modification date        */
 char    *copyright   = "Copyright 1997-2000 by Bradford L. Barrett";
 
 int     verbose      = 2;                     /* 2=verbose,1=err, 0=none  */ 
@@ -111,13 +112,15 @@ int     local_time   = 1;                     /* 1=localtime 0=GMT (UTC)  */
 int     ignore_hist  = 0;                     /* history flag (1=skip)    */
 int     hourly_graph = 1;                     /* hourly graph display     */
 int     hourly_stats = 1;                     /* hourly stats table       */
+int     daily_graph  = 1;                     /* daily graph display      */
+int     daily_stats  = 1;                     /* daily stats table        */
 int     ctry_graph   = 1;                     /* country graph display    */
 int     shade_groups = 1;                     /* Group shading 0=no 1=yes */
 int     hlite_groups = 1;                     /* Group hlite 0=no 1=yes   */
 int     mangle_agent = 0;                     /* mangle user agents       */
 int     incremental  = 0;                     /* incremental mode 1=yes   */
 int     use_https    = 0;                     /* use 'https://' on URL's  */
-int     visit_timeout= 3000;                  /* visit timeout (30 min)   */
+int     visit_timeout= 1800;                  /* visit timeout (seconds)  */
 int     graph_legend = 1;                     /* graph legend (1=yes)     */
 int     graph_lines  = 2;                     /* graph lines (0=none)     */
 int     fold_seq_err = 0;                     /* fold seq err (0=no)      */
@@ -171,7 +174,6 @@ int     cur_year=0, cur_month=0,              /* year/month/day/hour      */
 u_long  cur_tstamp=0;                         /* Timestamp...             */
 u_long  rec_tstamp=0;  
 u_long  req_tstamp=0;
-char    tmp_tstamp[16];                       /* used for ascii->float    */
 u_long  epoch;                                /* used for timestamp adj.  */
 
 int     check_dup=0;                          /* check for dup flag       */
@@ -237,6 +239,7 @@ int main(int argc, char *argv[])
    extern int opterr;
 
    time_t start_time, end_time;          /* program timers              */
+   float  temp_time;                     /* temporary time storage      */
    struct tms     mytms;                 /* bogus tms structure         */
 
    int    rec_year,rec_month=1,rec_day,rec_hour,rec_min,rec_sec;
@@ -255,16 +258,17 @@ int main(int argc, char *argv[])
                          "oct", "nov", "dec"};
 
    /* initalize epoch */
-   epoch=jdate(1,1,1990);                /* used for timestamp adj.     */
+   epoch=jdate(1,1,1970);                /* used for timestamp adj.     */
 
    /* add default index. alias */
    add_nlist("index.",&index_alias);
 
+   sprintf(tmp_buf,"%s/webalizer.conf",ETCDIR);
    /* check for default config file */
    if (!access("webalizer.conf",F_OK))
       get_config("webalizer.conf");
-   else if (!access("/etc/webalizer.conf",F_OK))
-      get_config("/etc/webalizer.conf");
+   else if (!access(tmp_buf,F_OK))
+      get_config(tmp_buf);
 
    /* get command line options */
    opterr = 0;     /* disable parser errors */
@@ -480,7 +484,7 @@ int main(int argc, char *argv[])
       else
       {
          /* Using DNS cache file <filaneme> */
-         if (verbose) printf("%s %s\n",msg_dns_usec,dns_cache);
+         if (verbose>1) printf("%s %s\n",msg_dns_usec,dns_cache);
       }
    }
 #endif  /* USE_DNS */
@@ -604,10 +608,10 @@ int main(int argc, char *argv[])
          /* Flag as a good one */
          good_rec = 1;
 
-         /* get current records timestamp (jdate-epochHHMMSS) */
+         /* get current records timestamp (seconds since epoch) */
          req_tstamp=cur_tstamp;
-         rec_tstamp=((jdate(rec_day,rec_month,rec_year)-epoch)*1000000)+
-            (rec_hour*10000) + (rec_min*100) + rec_sec;
+         rec_tstamp=((jdate(rec_day,rec_month,rec_year)-epoch)*86400)+
+                     (rec_hour*3600)+(rec_min*60)+rec_sec;
 
          /* Do we need to check for duplicate records? (incremental mode)   */
          if (check_dup)
@@ -640,7 +644,7 @@ int main(int argc, char *argv[])
          }
 
          /* check for out of sequence records */
-         if (rec_tstamp/10000 < cur_tstamp/10000)
+         if ( rec_tstamp+3600 < cur_tstamp)     /* give up to 1 hr of slop  */
          {
             if (!fold_seq_err) { total_ignore++; continue; }
             else
@@ -660,14 +664,6 @@ int main(int argc, char *argv[])
          /* DO SOME PRE-PROCESS FORMATTING            */
          /*********************************************/
 
-         /* lowercase hostname */
-         cp1 = log_rec.hostname;
-         while (*cp1 != '\0')
-         {
-            if ( (*cp1>='A') && (*cp1<='Z')) *cp1 += 'a'-'A';
-            cp1++;
-         }
-
          /* fix URL field */
          cp1 = cp2 = log_rec.url;
          /* handle null '-' case here... */
@@ -799,7 +795,7 @@ int main(int argc, char *argv[])
 		    if (mangle_agent<4)
 			if (*cp1>='0'&&*cp1<='9') *cp2++=*cp1++;
 		    if (mangle_agent<3)
-			while (*cp1!=';'&&*cp1!='\0'&&*cp1!=')') *cp2++=*cp1++;
+			while (*cp1!=';'&&*cp1!='\0'&&*cp1!='(') *cp2++=*cp1++;
 		    if (mangle_agent<2)
 		    {
 			/* Level 1 - try to get OS */
@@ -834,7 +830,7 @@ int main(int argc, char *argv[])
 		    if (mangle_agent<4)
 			if (*cp1>='0'&&*cp1<='9') *cp2++=*cp1++;
 		    if (mangle_agent<3)
-			while (*cp1!=' '&&*cp1!='\0'&&*cp1!=')') 
+			while (*cp1!=' '&&*cp1!='\0'&&*cp1!='(') 
 			    *cp2++=*cp1++;
 		    if (mangle_agent<2)
 		    {
@@ -866,7 +862,7 @@ int main(int argc, char *argv[])
 			if (mangle_agent<4)
 			    if (*cp1>='0'&&*cp1<='9') *cp2++=*cp1++;
 			if (mangle_agent<3)
-			    while (*cp1!=' '&&*cp1!='\0'&&*cp1!=')') 
+			    while (*cp1!=' '&&*cp1!='\0'&&*cp1!='(') 
 				*cp2++=*cp1++;
 			if (mangle_agent<2)
 			{
@@ -929,6 +925,8 @@ int main(int argc, char *argv[])
             while (*cp3>=32 && *cp3!='"') cp3++;
             *cp3='\0';
          }
+         /* unescape user name */
+         unescape(log_rec.ident);
 
          /********************************************/
          /* PROCESS RECORD                           */
@@ -992,11 +990,20 @@ int main(int argc, char *argv[])
          }
 #endif
 
+         /* lowercase hostname */
+         cp1 = log_rec.hostname;
+         while (*cp1 != '\0')
+         {
+            if ( (*cp1>='A') && (*cp1<='Z')) *cp1 += 'a'-'A';
+            cp1++;
+         }
+
          /* Ignore/Include check */
          if ( (isinlist(include_sites,log_rec.hostname)==NULL) &&
               (isinlist(include_urls,log_rec.url)==NULL)       &&
               (isinlist(include_refs,log_rec.refer)==NULL)     &&
-              (isinlist(include_agents,log_rec.agent)==NULL)   )
+              (isinlist(include_agents,log_rec.agent)==NULL)   &&
+              (isinlist(include_users,log_rec.ident)==NULL)    )
          {
             if (isinlist(ignored_sites,log_rec.hostname)!=NULL)
               { total_ignore++; continue; }
@@ -1006,6 +1013,8 @@ int main(int argc, char *argv[])
               { total_ignore++; continue; }
             if (isinlist(ignored_refs,log_rec.refer)!=NULL)
               { total_ignore++; continue; }
+            if (isinlist(ignored_users,log_rec.ident)!=NULL)
+              { total_ignore++; continue; }
          }
 
          /* Bump response code totals */
@@ -1307,11 +1316,15 @@ int main(int argc, char *argv[])
          }
          else if (total_bad) printf("(%lu %s) ",total_bad,msg_bad);
 
-         printf("%s %.2f %s", msg_in, (float)(end_time-start_time)/CLK_TCK,
-                                msg_seconds);
+         /* get processing time (end-start) */
+         temp_time = (float)(end_time-start_time)/CLK_TCK;
+         printf("%s %.2f %s", msg_in, temp_time, msg_seconds);
 
          /* calculate records per second */
-         i=((int)((float)total_rec / ((float)(end_time-start_time)/CLK_TCK)) );
+         if (temp_time)
+           i=( (int)( (float)total_rec/temp_time ) );
+         else i=0;
+
          if ( (i>0) && (i<=total_rec) ) printf(", %d/sec\n", i);
             else  printf("\n");
       }
@@ -1423,6 +1436,8 @@ void get_config(char *fname)
                      "DumpSearchStr",     /* Dump search str tab file   83  */
                      "DNSCache",          /* DNS Cache file name        84  */
                      "DNSChildren",       /* DNS Children (0=no DNS)    85  */
+                     "DailyGraph",        /* Daily Graph (0=no)         86  */
+                     "DailyStats"         /* Daily Stats (0=no)         87  */
                    };
 
    FILE *fp;
@@ -1564,6 +1579,8 @@ void get_config(char *fname)
         case 84: /* Disable DNSCache and DNSChildren if DNS is not enabled  */
         case 85: printf("%s '%s' (%s)\n",msg_bad_key,keyword,fname); break;
 #endif  /* USE_DNS */
+        case 86: daily_graph=(value[0]=='n')?0:1; break;  /* HourlyGraph    */
+        case 87: daily_stats=(value[0]=='n')?0:1; break;  /* HourlyStats    */
       }
    }
    fclose(fp);
@@ -1646,10 +1663,21 @@ void print_opts(char *pname)
 void print_version()
 {
  uname(&system_info);
- printf("Webalizer V%s-%s (%s %s) %s\n%s\n\n",
+ printf("Webalizer V%s-%s (%s %s) %s\n%s\n",
     version,editlvl,
     system_info.sysname,system_info.release,
     language,copyright);
+ if (debug_mode)
+ {
+    printf("Mod date: %s  Options: ",moddate);
+#ifdef USE_DNS
+    printf("DNS ");
+#else
+    printf("none");
+#endif
+    printf("\nDefault config dir: %s\n\n",ETCDIR);
+ }
+ else printf("\n");
  exit(1);
 }
 
@@ -1785,7 +1813,8 @@ void srch_string(char *ptr)
          if (*cp1=='+') *cp1=' ';                      /* change + to space  */
          if (sp_flg && *cp1==' ') { cp1++; continue; } /* compress spaces    */
          if (*cp1==' ') sp_flg=1; else sp_flg=0;       /* (flag spaces here) */
-         *cp2++=tolower(*cp1++);                       /* normal character   */
+         *cp2++=tolower(*cp1);                         /* normal character   */
+         cp1++;
       }
    }
    *cp2=0; cp2=tmpbuf;
